/*
 * File         : Choleski.java
 * Author       : Guansong Zhang
 * Created      : Sat Jan 24 17:29:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/04/21 21:14:10 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

import spmd.*;

class Choleski {

  static final int n=8;

  public static void main(String argv[]) {
    SpmdEnv spmd = new SpmdEnv(argv);

    Procs1 p = new Procs1(4);

    if (p.on()) {

      Range x = new Range(n);
      Range y = new CyclicRange(n, p.dim(0));

      Array2Float a = new Array2Float(x,y, p);

      Index ic = new Index(x);
      Index jc = new Index(y);

      ic.dim().over();
      for (ic.begin(); ic.test(); ic.next()) {
	jc.dim().over();
	for (jc.begin(); jc.test(); jc.next()) {
	  if (jc.idx()>=ic.idx())
	    a.data[a.element(ic,jc)]=ic.idx()+1;
	  else
	    a.data[a.element(ic,jc)]=jc.idx()+1;
	} jc.dim().revo();
      } ic.dim().revo();

      //      Spmd.rout.println("A = ");
      //      Adlib.aprintf("%7.3f%"+n+"N", a);

      Array1Float b = new Array1Float(x, p);

      Location i;
      Location j;

      i = x.location(0);
      j = y.location(0);

      if (i.at()) {
	if (j.at()) {
	  a.data[a.element(i,j)]= (float) Math.sqrt(a.data[a.element(i,j)]);
	} j.ta();
      } i.ta();
      
      for (int k=0; k<n-1; k++) {

	ic = new Index(x.subrng(n-k-1, k+1));
	ic.dim().over();
	for (ic.begin(); ic.test(); ic.next()) {
	  j = y.location(k);
	  if (j.at()) {
	    a.data[a.element(ic,j)] /= a.data[a.element(x.location(k),j)];
	  } j.ta();
	} ic.dim().revo();

	Adlib.remap(b.section(x.subrng(n-k-1,k+1)), 
		    a.section(x.subrng(n-k-1,k+1), y.location(k)));

	jc = new Index (y.subrng(n-k-1, k+1));
	jc.dim().over();
	for (jc.begin(); jc.test(); jc.next()) {

	  ic = new Index(x.subrng(n-k-1-jc.idx(), k+1+jc.idx()));
	  ic.dim().over();
	  for (ic.begin(); ic.test(); ic.next()) {

	    a.data[a.element(ic,jc)] -= b.data[b.element(ic)]*
	      b.data[b.element(x.location(k+1+jc.idx()))];

	  } ic.dim().revo();

	} jc.dim().revo();

	i = x.location(k+1);
	j = y.location(k+1);

	if (i.at()) {
	  if (j.at()) {
	    a.data[a.element(i,j)]= (float) Math.sqrt(a.data[a.element(i,j)]);
	  } j.ta();
	} i.ta();

// 	Adlib.rprintln(k+"th A = ");
// 	Adlib.aprintf("%7.3f%"+n+"N", a);

      }

      Adlib.rprintln("A = ");
      Adlib.aprintf("%7.3f%"+n+"N", a);
          
    } p.no();
 
    spmd.finalize();

  }
}
